clear all
clc

%Physical properties of the column
d=4.0;  %diameter of column, m
w=0.0078;   %wall thickness, m
h=7.0; %column height, m
qmh=681400;  %gas mass flowrate, kg/h
qm=qmh/3600;    %gas mass flowrate, kg/s

%defining grids and discretization
ny=240; %number of nodes in the y-direction;
nxw=3;  %number of nodes in the wall in the x-direction
dy=h/ny;    %step size in y-direction, discretization
dxw=w/nxw;  %step size in the x-direction (wall discretization)

tmax=60*100;    %time of adsorption, s
dt=0.025; %size of timestep, s
nt=tmax/dt; %number of timesteps

cA=(3.142*d^2)/4;   %cross sectin\onal area of column, m2
aA=0.62*cA; %area filled by the adsorbent, (packing factor is 0.62)
fA=cA-aA;   %area free for gas flow

%adsorbent properties
hfgas=@(xx) (4*10^-6)*(xx)^3 - 0.0007*(xx)^2 +2.6643*(xx) - 66.2325;
%enthalpy of gas as a function of temperature, kj/kg

%gas properties
gas_water_content=730*10^-6;    %water content in gas, mole fraction
mwgas=18.54;        %molecular weight of the gas
cpg=3000;   %heat capacity of gas, kj/kgk
cpgSI=3000; %heat capacity of the gas, j/kgk
mwg=zeros(ny,1);    %matrix for amount of water in gas
mwg1=mwg;   %dummy matrix to be used later
densgasf=@(tt) 58.94;   %density of gas as function of temperature, kg/m3
viscgasf=@(uu) 1.326E-05;   %viscousity of gas as function of temperature, PaS
condgasf=@(vv) 3.840E-02;   %thermal conductivity of gas as function of temperature, W/m2
% mgasentering=qm*dt;  %amount of gas entering the column each timestep, kg
tg=@(xx) (-2.53123*10^-13)*(xx)^5 + (4*10^-10)*(xx)^4 - (3.33*10^-7)*(xx)^3 + (9*10^-5)*(xx)^2 +...
    0.3745*(xx) + 25;   %temperature of gas as a function of enthalpy

%water properties
cpH2O=4.3;  %heat capacity of water, kj/kgk
mwwater=18.02;      %molecular weight of water

dtdivdy=dt/dy;  %constant to ease code

%wall properties
cpw=513;    %heat capacity of wall, j/kgk
kw=15.8;    %heat transfer coefficent of wall, W/mk
densw=7990; %density of wall material (steel), constant, at 400k
hwa=100;    %convection heat transfer coefficient, wall/adsorbent, W/m2k

%constant definitions

%wall constants
alpha=kw/(densw*cpw);
alphax=(alpha*dt)/dxw^2;    %must be larger than 0.25, stability reasons
alphay=(alpha*dt)/dy^2;     %must be larger than 0.25, stability reasons
if alphax>0.25  %reports if values gives unstable model
    display('ALPHAX TOO LARGE')
    display(alphax);
end
htx=(hwa*dt)/(dxw*cpw*densw);   %consatnt used in wall iteration
hxk=hwa*dxw/kw;  %constant used in wall iteration


%defining initial conditions
tw=27*ones(nxw,ny); %wall temperture matrix
tw1=tw; %dummy matrix for wall temperature
tgas=27*ones(ny,1); %gas temperature in column at start of regeneration
hgas=zeros(ny,1);   %enthalpy of gas in each section at start of regeneration
hgas1=hgas; %dummy matrix
tad=27*ones(ny,1);  %intial temperature of adsorbent
mgas=zeros(ny,1);   %initial amount of gas in column
mgas1=mgas; %dummy matrix

%%%CALCULATIONS%%%

wbar = waitbar(0,'Please wait...'); %defining the waitbar

for timestep=1:nt;  %timesteps
    t=dt*timestep;  %t is time from start, s
    minu=t*dt/60;
    qm=qmh/3600;      %feedgas flowrate, kg/s
    mgasentering=qm*dt;         %amount of gas entering the column each height section, kg
    mwgentering=mgasentering*gas_water_content*mwwater/mwgas;   %initial water content of gas

    
    %defining gas inlet properties
    if t<=tmax    %for time of adsorption
        tgasin=27;
        densgasinitial=densgasf(tgasin);
        hgasinitial=hfgas(tgasin);
        vginitial=qm/(fA*densgasinitial);
        hgasentering=hgasinitial*mgasentering;
    end
    
    tw=tw1; %dummy matrix (wall temperature at t=n+1, tw is at t=n)
    
    for j=1:ny; %iteration in the y-direction
        
         tw(:,1)=tw(:,2)+(tw(:,2)-tw(:,3));
        
        
        
        %%%GAS/ADSORPTION ITERATIONS
        
       tf=0.5*(tgas(j)+tad(j)); %arithmetic mean temperature of gas and adsorbent
q1=0.001*dt*dy*1402.28*qm*cpgSI*((cpgSI*viscgasf(tf)/condgasf(tf))^(-2/3))*((2.53*10^-4*qm/viscgasf(tf))...
    ^(-0.4069))*(tgas(j)-tad(j)); %heat transfer between gas and adsorbent from Geankoplis' relations (0.001 to convert
%from j to jk)
       qmax=0.995*(tgas(j)-tad(j))*(cpg*mgas(j)+cpH2O*mwg(j)); %max possible theoretical heat transfer (energy balance)
       q=min(q1,qmax);  %if q1>qmax, q must be qmax
       
       if q>0.001   %to avoid iteratons with too small values
           repeat=1;    %variable used to stay in the while-loop
           while repeat==1; %temperature iterations
               repeat=0;
               while tgas(j)-(q/(cpg*mgas(j)+cpH2O*mwg(j)))<tad(j); %if tgasnew<tad, the q must be reduced
                   q=0.9*((tgas(j)-tad(j))*(cpg*mgas(j)+cpH2O*mwg(j))); %new, reduced value of q
               end
               
               tgasnew=tgas(j)-(q/(cpg*mgas(j)+cpH2O*mwg(j))); %new gas temperature calculated from q
               C1=dmads*cpa*tad(j)+mwad(j)*(cpH2O*tad(j)-hvap-cpH2O*tgasnew)+q; %constant
               tadnew=tad(j)+(tgasnew-tad(j))*0.001; %random guess for new value of tad(j)
               error=1;
               counter=0;
               counter2=0;
               
               while error>0.0001 %iterations to make the guessed adsorption values correct
                   mwanew=polyval(mwadp,tadnew);
                   if tgasnew==tad(j)
                       mwanew=mwad(j)-0.005*mwad(j);
                   end
                   
                   if counter2>50 %to prevent iteration values jumping between two numbers
                       mwanew=0.8*mwanew2+0.2*mwanew;
                   end
                   

tadnew2=(C1+mwanew*(hvap+cpH2O*tgasnew))/(dmads*cpa+mwanew*cpH2O);
                   mwanew2=0.2*polyval(mwadp,tadnew2)+0.8*mwanew;
tadnew2=(C1+mwanew2*(hvap+cpH2O*tgasnew))/(dmads*cpa+mwanew*cpH2O);
                   error=abs(tadnew2-tadnew);
                   tadnew=(tadnew2+tadnew)/2;
                   counter=counter+1;
                   counter2=counter2+1;
                   
                   if counter>200
                       tadnew=((tadnew+tadnew2)+rand)/2;
                   end
                   
                   if tadnew<tad(j)
                       tadnew=tad(j)+(tgasnew-tad(j))*0.01*rand;
                   end
               end  %ends while-error
               
               if tadnew>tgasnew || tgasnew>tgas(j) || tadnew<tad(j) %checking for illogical values
                   q=0.90*q;
                   repeat=1;    %repeating itertions
                   display('illogical values')  %to check the illogical values
               end
           end  %end main iteration while repeat==1
           
           %defining new values
           deltam=mwad(j)-mwanew2;
           tgas(j)=tgasnew;
           tad(j)=tadnew;
           mwg(j)=mwg(j)+deltam;
           mwad(j)=mwanew2;
           hgas(j)=hgas(j)-q+(deltam*cpH2O*(tgas(j)-27));
           deltam=0;
       end  %end if q>0.001
    end  %end 'for' iteration in y-direction
    
    %wall temperatue in top layer of the column
    tw1(:,ny)=tw(:,ny-1);  %boundary condition wall: t unchaged outt
    
    %defining the inlet gas speed and H value
    vg(1)=qm/(fA*densgasinitial);   %gas velocity in first cell, layer, section
    movecells(1)=vginitial*dtdivdy; %how big part of cell 1 that will move upwards
    densgas(1)=densgasf(tgas(1));   %density of gas in first cell
    
    %calculations of gas movement upwards the column
    for i=2:ny
        densgas(i)=densgasf(tgas(i));   %desntiy of gas in cell i
        vg(i)=qm/(fA*densgas(i-1));     %velocity of gas in cell i
        movecells(i)=vg(i)*dtdivdy;     %how big part of cell i that will be pushed upwards
        
        if movecells(i)>1  %if so, we check if it might cause problems
            display('error, moving more than one cell per timestep')
            return
        end
        
        movegascells(i)=movecells(i)*mgas(i);   %how much mass in cell i moves upwarsds
        moveHcells(i)=movecells(i)*hgas(i);     %how much energy moves upwards
        movemwg(i)=movecells(i)*mwg(i);
        
    end
    
        movegascells(1)=movecells(1)*mgas(1);   %how much mass in cell 1 moves upwards
        moveHcells(1)=movecells(1)*hgas(1);     %how much energy in cell 1 moves upwards
        movemwg(1)=movecells(1)*mwg(1);
    
    for i=2:ny
        
        mgas1(i)=mgas(i)-movegascells(i)+movegascells(i-1); %amount of gas in cell i after it's moved
        hgas1(i)=hgas(i)-moveHcells(i)+moveHcells(i-1);     %amount of energy in gas in cell i after it's moved
        mwg1(i)=mwg(i)-movemwg(i)+movemwg(i-1);
        
    end
    
        mgas1(1)=mgas(1)-movegascells(1)+mgasentering;  %amount of gas in cell 1 after it's moved
        hgas1(1)=hgas(1)-moveHcells(1)+hgasentering;    %amount of energy in gas in cell 1 after it's moved
        mwg1(1)=mwg(1)-movemwg(1);
        mgas=mgas1; hgas=hgas1;
        mwg=mwg1;
        for i=1:ny
            if mgas(i)<0.001*mgasentering  %to avoid dividing by a number close to zero
                tgas(i)=27;
            else
                tgas(i)=tg(hgas(i)/mgas(i));
            end
        end
        
        %tgas(120);
        
        %end of gas movement
        
        waitbar(t/tmax, wbar, timestep*dt)
        
           tt=t/600;
        if ceil(tt) == floor(tt)    %creating matrices with temperature profiles every 10 mins
            i=t/600;
            heatdistmatrix(:,1,i)=tgas(:,1);
            heatdistmatrix(:,2,i)=tad(:,1);
            heatdistmatrix(:,3,i)=tw(nxw,1:ny);
        end
        
        minu=t/60;  %tracking gas outlet temperatures
        if ceil(minu) == floor(minu)
            tout(minu,1)=minu;
            tout(minu,2)=tgas(ny);
        end
        
end  %timestep ends,,,all calculations end at this stage
close(wbar)     %closes the waitbar

%creating plot of temperature profiles at t=tend
l=length(tgas);
sam=zeros(1,3);     %matrix for adsorption, gas and wall temperatures
sam(1:1,1)=tgas(1:1,1);
sam(1:1,2)=tad(1:1,1);
sam(1:1,3)=tw(nxw,1);
minutes=t/60;
aaa=(0:(7.0/(ny-1)):7.0);
plot(aaa, tw((nxw-1),1:ny), 'r', aaa, tgas(1:ny), 'g', aaa, tad(1:ny), 'b')
title(['temperature distribution at ', num2str(minutes),'minutes',', q=',num2str(qmh), 'kg/h'], 'fontweight','bold')
xlabel('Position of column [m]')
ylabel('Temperature [C]')
legend('Wall','Gas','Adsorbent')
axis([0 7.0 0 60])



